#!/usr/bin/env python

import numpy as np
import matplotlib.animation as animation
from matplotlib import pyplot as plt
from matplotlib.backends.qt_compat import QtWidgets
from matplotlib.widgets import Slider

π = np.pi

φ = np.linspace(0., 2 * π, 121)
cs = np.cos(φ)
sn = np.sin(φ)

N = 40

TT = np.arange(2 * N, -0.25, -0.5)
T = np.linspace(-5., 5., 2001)

Δt = 0.01
t = 0.

vs = vr = 0.

xs = 0.

LX = 10.
LY = 5.

DX = 6.

xlr = [-DX, DX]


def wavefronts(t):
    tt = (t + TT) % N
    res = np.array([(xs + (cs - vs) * τ + (vs - vr) * t, sn * τ) for τ in tt])
    # xm = np.abs(res[:, 0]) > X
    # ym = np.abs(res[:, 1]) > Y
    # res[:, 0][xm] = np.nan
    # res[:, 1][ym] = np.nan
    alpha = [(1.0 - τ / N)**2 for τ in tt]
    return res, alpha


fig, ax = plt.subplots(figsize=(16, 9))
manager = fig.canvas.manager
manager.set_window_title("Doppler")
manager.window.findChild(QtWidgets.QToolBar).setVisible(False)
manager.full_screen_toggle()

ax.set_xlim(-LX, LX)
ax.set_ylim(-LY, LY)
ax.set_xlabel('$x$')
ax.set_ylabel('$y$')
ax.set_aspect('equal')

waves, alphas = wavefronts(0.)
lines = [ax.plot(*wave, ('C0', 'C1')[i % 2], lw=4., alpha=alphas[i])[0] for i, wave in enumerate(waves)]
point = ax.plot(xs, 0., 'ko', ms=6.)[0]
ax.plot(xlr, [0., 0.], 'C3x', ms=6., zorder=2)[0]

plt.subplots_adjust(right=0.90, left=0.10, bottom=0.10, top=0.75)

taxes = fig.add_axes([0.15, 0.86, 0.30, 0.12]), fig.add_axes([0.55, 0.86, 0.30, 0.12])

for tax in taxes:
    tax.set_xlabel("$\\Delta t$ [s]")
    tax.set_xlim(T[0], T[-1])
    tax.set_ylim(-1.1, 1.1)
    tax.grid(True)

times = [tax.plot(T, np.cos(2 * π * T), 'C3-')[0] for tax in taxes]

vsax = fig.add_axes([0.90, 0.10, 0.03, 0.8])
vrax = fig.add_axes([0.96, 0.10, 0.03, 0.8])

sst = {
    'track_color': '0.7',
    'color': '0.7',
    'initcolor': 'k',
}

vs_slider = Slider(
    ax=vsax, label='$v_\\mathrm{źr}$ [$c$]', orientation='vertical', valmin=-2, valmax=2, valinit=0.0, valstep=0.1, **sst
)
vr_slider = Slider(
    ax=vrax, label='$v_\\mathrm{od}$ [$c$]', orientation='vertical', valmin=-2, valmax=2, valinit=0.0, valstep=0.1, **sst
)


def update(t):
    waves, alphas = wavefronts(t)
    for line, wave, alpha in zip(lines, waves, alphas):
        line.set_data(*wave)
        line.set_alpha(alpha)
    x = xs + (vs - vr) * t
    point.set_xdata(x)
    for x, time in zip(xlr, times):
        if np.abs(vs) < 1.:
            s = np.sign(x - xs)
            sr = np.sign(1 - s * vr)
            time.set_ydata(np.cos(2 * π * ((1 - s * vr) * (T + sr * t) - s * x) / (1 - s * vs)))
        else:
            time.set_ydata(np.nan * T)


def on_slider(val=None):
    global vs, vr, t, xs
    t = 0.
    vs = vs_slider.val
    vr = vr_slider.val
    # xs = 8. if vs < vr else -8. if vs > vr else 0.
    update(t)
    fig.canvas.draw_idle()


vs_slider.on_changed(on_slider)
vr_slider.on_changed(on_slider)


def animate(i):
    global t
    t += Δt
    update(t)
    return lines + [point] + times


anim = animation.FuncAnimation(fig, animate, interval=1, blit=True, cache_frame_data=False)

stop = False


def on_key(ev):
    global stop, xs, t
    if ev.key == ' ':
        stop = not stop
        if stop: anim.pause()
        else: anim.resume()
    else:
        if ev.key == 'left':
            xs -= 1.
        elif ev.key == 'right':
            xs += 1.
        else:
            return
        t = 0.
        update(t)
        fig.canvas.draw_idle()


fig.canvas.mpl_connect('key_press_event', on_key)

plt.show()
